class Dataset:
    @staticmethod
    def create(
        project,
        name,
        format,
        stage,
        data_url,
        annotations_url,
        credentials,
        upload,
        num_datapoints,
    ):
        if name is None or data_url is None or annotations_url is None:
            return None

        if format is not None:
            assert format in ("COCO",)
        else:
            format = "COCO"

        if stage is not None:
            assert stage in ("train", "validation", "test")
        else:
            stage = "test"

        session = project._session

        response = session._put(
            f"api/dataset/undefined/{project.id}",
            json={
                "name": name,
                "stage": stage,
                "data_url": data_url,
                "annotations_url": annotations_url,
                "access_token": credentials,
                "upload": upload,
                "projectId": project.id,
                "num_samples": num_datapoints,
            },
        )
        dataset_id = response["id"]

        if upload:
            endpoint = f"api/dataset/{dataset_id}/upload"
            session._upload(annotations_url, endpoint)
            session._upload(data_url, endpoint)
            session._post(endpoint, json={"num_samples": num_datapoints})

        return Dataset(
            project, dataset_id, name, format, stage, data_url, annotations_url
        )

    def __init__(self, project, id, name, format, stage, data_url, annotations_url):
        self.project = project
        self.id = id
        self.name = name
        self.format = format
        self.stage = stage
        self.data_url = data_url
        self.annotations_url = annotations_url

    def __repr__(self):
        res = f"{self.__module__}.{self.__class__.__name__}("
        res += f"\n  id={self.id}"
        res += f"\n  name={self.name}"
        res += f"\n  format={self.format}"
        res += f"\n  stage={self.stage}"
        res += f"\n  data_url={self.data_url}"
        res += f"\n  annotations_url={self.annotations_url}"
        res += f"\n)"
        return res

    def delete(self):
        self.project._session._delete(f"api/dataset/{self.id}/{self.project.id}")



import argparse

from efemarai import Session

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--config-file",
        "-c",
        default=None,
        help="Path where to store SDK configuration.",
    )
    args = parser.parse_args()

    Session._user_setup(config_file=args.config_file)

if __name__ == "__main__":
    main()



from efemarai.dataset import Dataset
from efemarai.domain import Domain
from efemarai.model import Model, ModelParams, ModelRepository
from efemarai.problem_type import ProblemType
from efemarai.stress_test import StressTest


class Project:
    @staticmethod
    def create(session, name, description, problem_type):
        if name is None or not ProblemType.has(problem_type):
            return None

        response = session._put(
            "api/project",
            json={
                "name": name,
                "description": description,
                "problem_type": problem_type,
            },
        )
        return Project(session, response["id"], name, description, problem_type)

    def __init__(self, session, id, name, description, problem_type):
        self._session = session
        self.id = id
        self.name = name
        self.description = description
        self.problem_type = ProblemType(problem_type)

    def __repr__(self):
        res = f"{self.__module__}.{self.__class__.__name__}("
        res += f"\n  id={self.id}"
        res += f"\n  name={self.name}"
        res += f"\n  description={self.description}"
        res += f"\n  problem_type={self.problem_type}"
        res += f"\n)"
        return res

    @property
    def models(self):
        return [
            Model(
                self,
                model["id"],
                model["name"],
                repository=ModelRepository(
                    url=model["repository_url"],
                    branch=model["branch"],
                    access_token=model["access_token"],
                ),
                params=ModelParams(url=model["model_url"]),
            )
            for model in self._session._get(f"api/models/{self.id}")
        ]

    def model(self, name, repository=None, params=None, **kwargs):
        model = next((m for m in self.models if m.name == name), None)

        if model is None:
            model = Model.create(self, name, repository, params)

        return model

    @property
    def datasets(self):
        return [
            Dataset(
                self,
                dataset["id"],
                dataset["name"],
                dataset["format"],
                dataset["stage"],
                dataset["data_url"],
                dataset["annotations_url"],
            )
            for dataset in self._session._get(f"api/datasets/{self.id}")
        ]

    def dataset(
        self,
        name,
        format=None,
        stage=None,
        data_url=None,
        annotations_url=None,
        credentials=None,
        upload=False,
        num_datapoints=None,
        **kwargs,
    ):
        dataset = next((d for d in self.datasets if d.name == name), None)

        if dataset is None:
            dataset = Dataset.create(
                self,
                name,
                format,
                stage,
                data_url,
                annotations_url,
                credentials,
                upload,
                num_datapoints,
            )

        return dataset

    @property
    def domains(self):
        return [
            Domain(
                self,
                domain["id"],
                domain["name"],
                domain["transformations"],
                domain["graph"],
            )
            for domain in self._session._get(f"api/domains/{self.id}")
        ]

    def domain(self, name, transformations=None, graph=None, **kwargs):
        domain = next((d for d in self.domains if d.name == name), None)

        if domain is None:
            domain = Domain.create(self, name, transformations, graph)

        return domain

    @property
    def stress_tests(self):
        return [
            StressTest(
                self,
                test["id"],
                test["name"],
                test["model"]["$oid"],
                test["domain"]["$oid"],
                test["original_dataset"]["$oid"],
                test["states"][-1]["name"],
            )
            for test in self._session._get(f"api/getRuns/{self.id}")["objects"]
        ]

    def stress_test(
        self,
        name,
        model=None,
        domain=None,
        dataset=None,
        num_samples=None,
        num_runs=None,
        concurrent_runs=None,
        **kwargs,
    ):
        test = next((t for t in self.stress_tests if t.name == name), None)

        if test is None:
            test = StressTest.create(
                self,
                name,
                model,
                domain,
                dataset,
                num_samples,
                num_runs,
                concurrent_runs,
            )

        return test

    def delete(self):
        for domain in self.domains:
            domain.delete()

        for dataset in self.datasets:
            dataset.delete()

        for model in self.models:
            model.delete()

        self._session._delete(f"api/project/{self.id}")



import os
import zipfile

from enum import Enum
from time import sleep

import boto3
from appdirs import user_data_dir
from botocore.errorfactory import ClientError

from rich.console import Console
from rich.prompt import Confirm, Prompt

from efemarai.problem_type import ProblemType

console = Console()


class TestRunState(Enum):
    NotStarted = "NotStarted"
    GeneratingBaselines = "GeneratingBaselines"
    Starting = "Starting"
    Running = "Running"
    GeneratingReport = "Generating Report"
    Finished = "Finished"
    Failed = "Failed"


class StressTest:
    @staticmethod
    def create(
        project,
        name,
        model,
        domain,
        dataset,
        num_samples=50,
        num_runs=2,
        concurrent_runs=1,
    ):
        if isinstance(model, str):
            model = project.model(model)

        if isinstance(dataset, str):
            dataset = project.dataset(dataset)

        if isinstance(domain, str):
            domain = project.domain(domain)

        if num_samples is None:
            num_samples = 50

        if num_runs is None:
            num_runs = 1

        if concurrent_runs is None:
            concurrent_runs = 1

        response = project._session._post(
            "api/runTest",
            json={
                "name": name,
                "model": model.id,
                "dataset": dataset.id,
                "domain": domain.id,
                "project": project.id,
                "samples_per_run": num_samples,
                "run_count": num_runs,
                "concurrent_runs": concurrent_runs,
            },
        )
        return StressTest(
            project, response["id"], name, model, domain, dataset, "NotStarted"
        )

    def __init__(self, project, id, name, model, domain, dataset, state):
        self.project = project
        self.id = id
        self.name = name

        if isinstance(model, str):
            self._model = None
            self._model_id = model
        else:
            self._model = model
            self._model_id = model.id

        if isinstance(domain, str):
            self._domain = None
            self._domain_id = domain
        else:
            self._domain = domain
            self._domain_id = domain.id

        if isinstance(dataset, str):
            self._dataset = None
            self._dataset_id = dataset
        else:
            self._dataset = dataset
            self._dataset_id = dataset.id

        self.state = TestRunState(state)

    @property
    def model(self):
        if self._model is None:
            self._model = next(m for m in self.project.models if m.id == self._model_id)
        return self._model

    @property
    def domain(self):
        if self._domain is None:
            self._domain = next(
                d for d in self.project.domains if d.id == self._domain__id
            )
        return self._domain

    @property
    def dataset(self):
        if self._dataset is None:
            self._dataset = next(
                d for d in self.project.datasets if d.id == self._dataset_id
            )
        return self._dataset

    def vulnerabilities_dataset(
        self,
        min_score=0.0,
        include_dataset=False,
        path=None,
        unzip=True,
        ignore_cache=False,
    ):
        if self.state != TestRunState.Finished:
            console.print(
                (
                    ":warning: Cannot export vulnerabilities "
                    "dataset as stress test is still running"
                ),
                style="yellow",
            )
            return None

        if path is None:
            path = user_data_dir(appname="efemarai")

        path = os.path.join(path, self.id)

        export_format = None
        if self.project.problem_type == ProblemType.Classification:
            export_format = "imagenet"
        elif self.project.problem_type == ProblemType.ObjectDetection:
            export_format = "coco"
        elif self.project.problem_type == ProblemType.InstanceSegmentation:
            export_format = "coco"

        if export_format is None:
            console.print(":poop: Unsupported problem type.", style="red")
            return None

        if not ignore_cache:
            name = "vulnerabilities_dataset"
            name += f"_{export_format}_{include_dataset}_{min_score:.3f}"
            cache_name = os.path.join(path, name)
            if os.path.exists(cache_name) or os.path.exists(cache_name + ".zip"):
                return cache_name

        access = self.project._session._post(
            "api/exportDataset",
            json={
                "id": self.id,
                "format": export_format,
                "merge": include_dataset,
                "min_score": min_score,
                "async_download": True,
            },
        )

        s3 = boto3.client(
            "s3",
            aws_access_key_id=access["AccessKeyId"],
            aws_secret_access_key=access["SecretAccessKey"],
            aws_session_token=access["SessionToken"],
            endpoint_url=access["Url"],
        )

        with console.status(f"Generating '{self.name}' vulnerabilities dataset"):
            while True:
                try:
                    response = s3.head_object(
                        Bucket=access["Bucket"], Key=access["ObjectKey"]
                    )
                    size = response["ContentLength"]
                    break
                except ClientError:
                    sleep(1)

        with self.project._session._progress_bar() as progress:
            task = progress.add_task(f"Downloading dataset ", total=float(size))
            callback = lambda num_bytes: progress.advance(task, num_bytes)

            os.makedirs(path, exist_ok=True)
            filename = os.path.join(path, os.path.basename(access["ObjectKey"]))

            s3.download_file(
                access["Bucket"], access["ObjectKey"], filename, Callback=callback
            )

        if unzip:
            with console.status(f"Unzipping dataset"):
                dirname = os.path.splitext(filename)[0]
                with zipfile.ZipFile(filename, "r") as f:
                    f.extractall(dirname)

                os.remove(filename)

                filename = dirname

        console.print(
            (
                f":heavy_check_mark: Downloaded '{self.name}' "
                f"vulnerabilities dataset to \n  {filename}"
            ),
            style="green",
        )

        return filename



from bson.objectid import ObjectId

class Domain:
    @staticmethod
    def create(project, name, transformations, graph):
        if name is None or name is None or transformations is None or graph is None:
            return None

        session = project._session
        response = session._put(
            f"api/domain/undefined/{project.id}",
            json={"name": name, "projectId": project.id},
        )
        domain_id = response["id"]

        operators = {
            operator["name"]: operator
            for operator in session._get("api/getDefaultOperators")
        }

        lookup_transformation_id = {}
        domain_transformations = []
        for transformation in transformations:
            tf = {
                "_id": {"$oid": str(ObjectId())},
                "name": transformation["name"],
                "operator": operators[transformation["operator"]]["_id"]["$oid"],
                "axes": transformation["axes"],
                "category": operators[transformation["operator"]]["category"],
            }
            lookup_transformation_id[tf["name"]] = tf["_id"]["$oid"]
            domain_transformations.append(tf)

        domain_graph = []
        for a, b in graph:
            domain_graph.append(
                {
                    "previous": lookup_transformation_id[a],
                    "next": lookup_transformation_id[b],
                }
            )

        session._post(
            f"api/saveDomainFlow",
            json={
                "_id": domain_id,
                "transformations": domain_transformations,
                "graph": domain_graph,
            },
        )

        return Domain(project, domain_id, name, domain_transformations, domain_graph)

    def __init__(self, project, id, name, transformations, graph):
        self.project = project
        self.id = id
        self.name = name
        self.transformations = transformations
        self.graph = graph

    def __repr__(self):
        res = f"{self.__module__}.{self.__class__.__name__}("
        res += f"\n  id={self.id}"
        res += f"\n  name={self.name}"
        res += f"\n  transformations={self.transformations}"
        res += f"\n  graph={self.graph}"
        res += f"\n)"
        return res

    def delete(self):
        self.project._session._delete(f"api/domain/{self.id}/{self.project.id}")
